@@ -26,6 +26,7 @@
 #include <util/alloc.h>
 #include <util/gv_ctype.h>
 #include <util/gv_math.h>
+#include <util/path.h>
 #include <util/startswith.h>
 #include <util/strcasecmp.h>
 #include <util/streq.h>
@@ -35,7 +36,7 @@
 int late_int(void *obj, attrsym_t *attr, int defaultValue, int minimum) {
     if (attr == NULL)
         return defaultValue;
-    char *p = ag_xget(obj, attr);
+    char *p = agxget(obj, attr);
     if (!p || p[0] == '\0')
         return defaultValue;
     char *endp;
@@ -51,7 +52,7 @@ double late_double(void *obj, attrsym_t *attr, double defaultValue,
                    double minimum) {
     if (!attr || !obj)
         return defaultValue;
-    char *p = ag_xget(obj, attr);
+    char *p = agxget(obj, attr);
     if (!p || p[0] == '\0')
         return defaultValue;
     char *endp;
@@ -162,12 +163,12 @@ pointf coord(node_t * n)
 #define W_DEGREE 5
 
 /*
- *	Evaluate a Bezier curve at a particular parameter value
+ *	Evaluate a Bézier curve at a particular parameter value
  *      Fill in control points for resulting sub-curves if "Left" and
  *	"Right" are non-null.
  *
  */
-pointf Bezier(pointf *V, double t, pointf *Left, pointf *Right) {
+pointf Bezier(const pointf *V, double t, pointf *Left, pointf *Right) {
     const int degree = 3;
     int i, j;			/* Index variables      */
     pointf Vtemp[W_DEGREE + 1][W_DEGREE + 1];
@@ -249,7 +250,8 @@ static char *findPath(const strview_t *dirs, const char *str) {
     static agxbuf safefilename;
 
     for (const strview_t *dp = dirs; dp != NULL && dp->data != NULL; dp++) {
-	agxbprint(&safefilename, "%.*s%s%s", (int)dp->size, dp->data, DIRSEP, str);
+	agxbprint(&safefilename, "%.*s%c%s", (int)dp->size, dp->data, PATH_SEPARATOR,
+	          str);
 	char *filename = agxbuse(&safefilename);
 	if (access(filename, R_OK) == 0)
 	    return filename;
@@ -302,7 +304,7 @@ const char *safefile(const char *filename)
 	    dirs = mkDirlist(pathlist);
     }
 
-    if (*filename == DIRSEP[0] || !dirs)
+    if (*filename == PATH_SEPARATOR || !dirs)
 	return filename;
 
     return findPath(dirs, filename);
@@ -366,8 +368,8 @@ pointf dotneato_closest(splines * spl, pointf pt)
     }
 
     bz = spl->list[besti];
-    /* Pick best Bezier. If bestj is the last point in the B-spline, decrement.
-     * Then set j to be the first point in the corresponding Bezier by dividing
+    /* Pick best Bézier. If bestj is the last point in the B-spline, decrement.
+     * Then set j to be the first point in the corresponding Bézier by dividing
      * then multiplying be 3. Thus, 0,1,2 => 0; 3,4,5 => 3, etc.
      */
     if (bestj == bz.size-1)
@@ -434,11 +436,10 @@ void common_init_node(node_t * n)
     fi.fontsize = late_double(n, N_fontsize, DEFAULT_FONTSIZE, MIN_FONTSIZE);
     fi.fontname = late_nnstring(n, N_fontname, DEFAULT_FONTNAME);
     fi.fontcolor = late_nnstring(n, N_fontcolor, DEFAULT_COLOR);
-    ND_label(n) = make_label(n, str,
-	        (aghtmlstr(str) ? LT_HTML : LT_NONE) | ( (shapeOf(n) == SH_RECORD) ? LT_RECD : LT_NONE),
+    ND_label(n) = make_label(n, str, aghtmlstr(str), shapeOf(n) == SH_RECORD,
 		fi.fontsize, fi.fontname, fi.fontcolor);
     if (N_xlabel && (str = agxget(n, N_xlabel)) && str[0]) {
-	ND_xlabel(n) = make_label(n, str, aghtmlstr(str) ? LT_HTML : LT_NONE,
+	ND_xlabel(n) = make_label(n, str, aghtmlstr(str), false,
 				fi.fontsize, fi.fontname, fi.fontcolor);
 	GD_has_labels(agraphof(n)) |= NODE_XLABEL;
     }
@@ -513,7 +514,7 @@ void common_init_edge(edge_t *e) {
     lfi.fontname = NULL;
     if (E_label && (str = agxget(e, E_label)) && str[0]) {
 	initFontEdgeAttr(e, &fi);
-	ED_label(e) = make_label(e, str, aghtmlstr(str) ? LT_HTML : LT_NONE,
+	ED_label(e) = make_label(e, str, aghtmlstr(str), false,
 				fi.fontsize, fi.fontname, fi.fontcolor);
 	GD_has_labels(sg) |= EDGE_LABEL;
 	ED_label_ontop(e) = mapbool(late_string(e, E_label_float, "false"));
@@ -522,21 +523,21 @@ void common_init_edge(edge_t *e) {
     if (E_xlabel && (str = agxget(e, E_xlabel)) && str[0]) {
 	if (!fi.fontname)
 	    initFontEdgeAttr(e, &fi);
-	ED_xlabel(e) = make_label(e, str, aghtmlstr(str) ? LT_HTML : LT_NONE,
+	ED_xlabel(e) = make_label(e, str, aghtmlstr(str), false,
 				fi.fontsize, fi.fontname, fi.fontcolor);
 	GD_has_labels(sg) |= EDGE_XLABEL;
     }
 
     if (E_headlabel && (str = agxget(e, E_headlabel)) && str[0]) {
 	initFontLabelEdgeAttr(e, &fi, &lfi);
-	ED_head_label(e) = make_label(e, str, aghtmlstr(str) ? LT_HTML : LT_NONE,
+	ED_head_label(e) = make_label(e, str, aghtmlstr(str), false,
 				lfi.fontsize, lfi.fontname, lfi.fontcolor);
 	GD_has_labels(sg) |= HEAD_LABEL;
     }
     if (E_taillabel && (str = agxget(e, E_taillabel)) && str[0]) {
 	if (!lfi.fontname)
 	    initFontLabelEdgeAttr(e, &fi, &lfi);
-	ED_tail_label(e) = make_label(e, str, aghtmlstr(str) ? LT_HTML : LT_NONE,
+	ED_tail_label(e) = make_label(e, str, aghtmlstr(str), false,
 				lfi.fontsize, lfi.fontname, lfi.fontcolor);
 	GD_has_labels(sg) |= TAIL_LABEL;
     }
