@@ -18,6 +18,8 @@
 #include <stddef.h>
 #include <stdbool.h>
 #include <util/alloc.h>
+#include <util/overflow.h>
+#include <util/unreachable.h>
 
 static size_t size_of_matrix_type(int type){
   size_t size = 0;
@@ -34,12 +36,8 @@ static size_t size_of_matrix_type(int type){
   case MATRIX_TYPE_PATTERN:
     size = 0;
     break;
-  case MATRIX_TYPE_UNKNOWN:
-    size = 0;
-    break;
   default:
-    size = 0;
-    break;
+    UNREACHABLE();
   }
 
   return size;
@@ -126,12 +124,8 @@ SparseMatrix SparseMatrix_transpose(SparseMatrix A){
       }
     }
     break;
-  case MATRIX_TYPE_UNKNOWN:
-    SparseMatrix_delete(B);
-    return NULL;
   default:
-    SparseMatrix_delete(B);
-    return NULL;
+    UNREACHABLE();
   }
 
 
@@ -252,12 +246,8 @@ bool SparseMatrix_is_symmetric(SparseMatrix A, bool test_pattern_symmetry_only)
     }
     res = true;
     break;
-  case MATRIX_TYPE_UNKNOWN:
-    goto RETURN;
-    break;
   default:
-    goto RETURN;
-    break;
+    UNREACHABLE();
   }
 
   if (!test_pattern_symmetry_only) {
@@ -293,7 +283,7 @@ static SparseMatrix SparseMatrix_init(int m, int n, int type, size_t sz, int for
   return A;
 }
 
-static SparseMatrix SparseMatrix_alloc(SparseMatrix A, int nz){
+static void SparseMatrix_alloc(SparseMatrix A, int nz) {
   int format = A->format;
   size_t nz_t = (size_t) nz; /* size_t is 64 bit on 64 bit machine. Using nz*A->size can overflow. */
 
@@ -313,7 +303,6 @@ static SparseMatrix SparseMatrix_alloc(SparseMatrix A, int nz){
     break;
   }
   A->nzmax = nz;
-  return A;
 }
 
 static SparseMatrix SparseMatrix_realloc(SparseMatrix A, int nz){
@@ -357,11 +346,16 @@ SparseMatrix SparseMatrix_new(int m, int n, int nz, int type, int format){
   sz = size_of_matrix_type(type);
   A = SparseMatrix_init(m, n, type, sz, format);
 
-  if (nz > 0) A = SparseMatrix_alloc(A, nz);
+  if (nz > 0) SparseMatrix_alloc(A, nz);
   return A;
 
 }
-SparseMatrix SparseMatrix_general_new(int m, int n, int nz, int type, size_t sz, int format){
+
+/// a generalized version of `SparseMatrix_new`
+///
+/// Allows elements to be any data structure, not just real/int/complex etc
+static SparseMatrix SparseMatrix_general_new(int m, int n, int nz, int type,
+                                             size_t sz, int format) {
   /* return a sparse matrix skeleton with row dimension m and storage nz. If nz == 0, 
      only row pointers are allocated. this is more general and allow elements to be 
      any data structure, not just real/int/complex etc
@@ -370,7 +364,7 @@ SparseMatrix SparseMatrix_general_new(int m, int n, int nz, int type, size_t sz,
 
   A = SparseMatrix_init(m, n, type, sz, format);
 
-  if (nz > 0) A = SparseMatrix_alloc(A, nz);
+  if (nz > 0) SparseMatrix_alloc(A, nz);
   return A;
 
 }
@@ -399,10 +393,8 @@ static void SparseMatrix_export_csr(FILE *f, SparseMatrix A){
   case MATRIX_TYPE_PATTERN:
     fprintf(f,"%%%%MatrixMarket matrix coordinate pattern general\n");
     break;
-  case MATRIX_TYPE_UNKNOWN:
-    return;
   default:
-    return;
+    UNREACHABLE();
   }
 
   fprintf(f,"%d %d %d\n",A->m,A->n,A->nz);
@@ -443,20 +435,13 @@ static void SparseMatrix_export_csr(FILE *f, SparseMatrix A){
       }
     }
     break;
-  case MATRIX_TYPE_UNKNOWN:
-    return;
   default:
-    return;
+    UNREACHABLE();
   }
 
 }
 
 static void SparseMatrix_export_coord(FILE *f, SparseMatrix A){
-  int *ia, *ja;
-  double *a;
-  int *ai;
-  int i;
-  
   switch (A->type){
   case MATRIX_TYPE_REAL:
     fprintf(f,"%%%%MatrixMarket matrix coordinate real general\n");
@@ -470,44 +455,42 @@ static void SparseMatrix_export_coord(FILE *f, SparseMatrix A){
   case MATRIX_TYPE_PATTERN:
     fprintf(f,"%%%%MatrixMarket matrix coordinate pattern general\n");
     break;
-  case MATRIX_TYPE_UNKNOWN:
-    return;
   default:
-    return;
+    UNREACHABLE();
   }
 
   fprintf(f,"%d %d %d\n",A->m,A->n,A->nz);
-  ia = A->ia;
-  ja = A->ja;
-  a = A->a;
+  const int *const ia = A->ia;
+  const int *const ja = A->ja;
   switch (A->type){
-  case MATRIX_TYPE_REAL:
-    a = A->a;
-    for (i = 0; i < A->nz; i++){
+  case MATRIX_TYPE_REAL: {
+    const double *const a = A->a;
+    for (int i = 0; i < A->nz; i++) {
       fprintf(f, "%d %d %16.8g\n",ia[i]+1, ja[i]+1, a[i]);
     }
     break;
-  case MATRIX_TYPE_COMPLEX:
-    a = A->a;
-    for (i = 0; i < A->nz; i++){
+  }
+  case MATRIX_TYPE_COMPLEX: {
+    const double *const a = A->a;
+    for (int i = 0; i < A->nz; i++) {
       fprintf(f, "%d %d %16.8g %16.8g\n",ia[i]+1, ja[i]+1, a[2*i], a[2*i+1]);
     }
     break;
-  case MATRIX_TYPE_INTEGER:
-    ai = A->a;
-    for (i = 0; i < A->nz; i++){
+  }
+  case MATRIX_TYPE_INTEGER: {
+    const int *const ai = A->a;
+    for (int i = 0; i < A->nz; i++) {
       fprintf(f, "%d %d %d\n",ia[i]+1, ja[i]+1, ai[i]);
     }
     break;
   case MATRIX_TYPE_PATTERN:
-    for (i = 0; i < A->nz; i++){
+    for (int i = 0; i < A->nz; i++){
       fprintf(f, "%d %d\n",ia[i]+1, ja[i]+1);
     }
     break;
-  case MATRIX_TYPE_UNKNOWN:
-    return;
+  }
   default:
-    return;
+    UNREACHABLE();
   }
 }
 
@@ -523,7 +506,7 @@ void SparseMatrix_export(FILE *f, SparseMatrix A){
     SparseMatrix_export_coord(f, A);
     break;
   default:
-    assert(0);
+    UNREACHABLE();
   }
 }
 
@@ -584,8 +567,6 @@ static SparseMatrix SparseMatrix_from_coordinate_arrays_internal(int nz, int m,
 
   if (m <=0 || n <= 0 || nz < 0) return NULL;
   A = SparseMatrix_general_new(m, n, nz, type, sz, FORMAT_CSR);
-  assert(A);
-  if (!A) return NULL;
   ia = A->ia;
   ja = A->ja;
 
@@ -599,8 +580,7 @@ static SparseMatrix SparseMatrix_from_coordinate_arrays_internal(int nz, int m,
     a = A->a;
     for (i = 0; i < nz; i++){
       if (irn[i] < 0 || irn[i] >= m || jcn[i] < 0 || jcn[i] >= n) {
-	assert(0);
-	return NULL;
+	UNREACHABLE();
       }
       ia[irn[i]+1]++;
     }
@@ -618,8 +598,7 @@ static SparseMatrix SparseMatrix_from_coordinate_arrays_internal(int nz, int m,
     a = A->a;
     for (i = 0; i < nz; i++){
       if (irn[i] < 0 || irn[i] >= m || jcn[i] < 0 || jcn[i] >= n) {
-	assert(0);
-	return NULL;
+	UNREACHABLE();
       }
       ia[irn[i]+1]++;
     }
@@ -638,8 +617,7 @@ static SparseMatrix SparseMatrix_from_coordinate_arrays_internal(int nz, int m,
     ai = A->a;
     for (i = 0; i < nz; i++){
       if (irn[i] < 0 || irn[i] >= m || jcn[i] < 0 || jcn[i] >= n) {
-	assert(0);
-	return NULL;
+	UNREACHABLE();
       }
       ia[irn[i]+1]++;
     }
@@ -655,8 +633,7 @@ static SparseMatrix SparseMatrix_from_coordinate_arrays_internal(int nz, int m,
   case MATRIX_TYPE_PATTERN:
     for (i = 0; i < nz; i++){
       if (irn[i] < 0 || irn[i] >= m || jcn[i] < 0 || jcn[i] >= n) {
-	assert(0);
-	return NULL;
+	UNREACHABLE();
       }
       ia[irn[i]+1]++;
     }
@@ -667,25 +644,8 @@ static SparseMatrix SparseMatrix_from_coordinate_arrays_internal(int nz, int m,
     for (i = m; i > 0; i--) ia[i] = ia[i - 1];
     ia[0] = 0;
     break;
-  case MATRIX_TYPE_UNKNOWN:
-    for (i = 0; i < nz; i++){
-      if (irn[i] < 0 || irn[i] >= m || jcn[i] < 0 || jcn[i] >= n) {
-	assert(0);
-	return NULL;
-      }
-      ia[irn[i]+1]++;
-    }
-    for (i = 0; i < m; i++) ia[i+1] += ia[i];
-    memcpy(A->a, val0, A->size*((size_t)nz));
-    for (i = 0; i < nz; i++){
-      ja[ia[irn[i]]++] = jcn[i];
-    }
-    for (i = m; i > 0; i--) ia[i] = ia[i - 1];
-    ia[0] = 0;
-    break;
   default:
-    assert(0);
-    return NULL;
+    UNREACHABLE();
   }
   A->nz = nz;
 
@@ -724,7 +684,6 @@ SparseMatrix SparseMatrix_add(SparseMatrix A, SparseMatrix B){
   nzmax = A->nz + B->nz;/* just assume that no entries overlaps for speed */
 
   C = SparseMatrix_new(m, n, nzmax, A->type, FORMAT_CSR);
-  if (!C) goto RETURN;
   ic = C->ia;
   jc = C->ja;
 
@@ -826,14 +785,11 @@ SparseMatrix SparseMatrix_add(SparseMatrix A, SparseMatrix B){
     }
     break;
   }
-  case MATRIX_TYPE_UNKNOWN:
-    break;
   default:
-    break;
+    UNREACHABLE();
   }
   C->nz = nz;
 
- RETURN:
   free(mask);
 
   return C;
@@ -919,8 +875,7 @@ void SparseMatrix_multiply_vector(SparseMatrix A, double *v, double **res) {
     }
     break;
   default:
-    assert(0);
-    u = NULL;
+    UNREACHABLE();
   }
   *res = u;
 
@@ -956,14 +911,13 @@ SparseMatrix SparseMatrix_multiply(SparseMatrix A, SparseMatrix B){
       jj = ja[j];
       for (k = ib[jj]; k < ib[jj+1]; k++){
 	if (mask[jb[k]] != -i - 2){
-	  if ((nz+1) <= nz) {
+	  if (sadd_overflow(nz, 1, &nz)) {
 #ifdef DEBUG_PRINT
 	    fprintf(stderr,"overflow in SparseMatrix_multiply !!!\n");
 #endif
 	    free(mask);
 	    return NULL;
 	  }
-	  nz++;
 	  mask[jb[k]] = -i - 2;
 	}
       }
@@ -971,7 +925,6 @@ SparseMatrix SparseMatrix_multiply(SparseMatrix A, SparseMatrix B){
   }
 
   C = SparseMatrix_new(m, B->n, nz, type, FORMAT_CSR);
-  if (!C) goto RETURN;
   ic = C->ia;
   jc = C->ja;
   
@@ -1073,16 +1026,12 @@ SparseMatrix SparseMatrix_multiply(SparseMatrix A, SparseMatrix B){
       ic[i+1] = nz;
     }
     break;
-  case MATRIX_TYPE_UNKNOWN:
   default:
-    SparseMatrix_delete(C);
-    C = NULL; goto RETURN;
-    break;
+    UNREACHABLE();
   }
   
   C->nz = nz;
 
- RETURN:
   free(mask);
   return C;
 }
@@ -1104,7 +1053,7 @@ SparseMatrix SparseMatrix_multiply3(SparseMatrix A, SparseMatrix B, SparseMatrix
 
   if (A->type != B->type || B->type != C->type){
 #ifdef DEBUG
-    printf("in SparseMatrix_multiply, the matrix types do not match, right now only multiplication of matrices of the same type is supported\n");
+    printf("in SparseMatrix_multiply3, the matrix types do not match, right now only multiplication of matrices of the same type is supported\n");
 #endif
     return NULL;
   }
@@ -1125,13 +1074,13 @@ SparseMatrix SparseMatrix_multiply3(SparseMatrix A, SparseMatrix B, SparseMatrix
 	ll = jb[l];
 	for (k = ic[ll]; k < ic[ll+1]; k++){
 	  if (mask[jc[k]] != -i - 2){
-	    if ((nz+1) <= nz) {
+	    if (sadd_overflow(nz, 1, &nz)) {
 #ifdef DEBUG_PRINT
-	      fprintf(stderr,"overflow in SparseMatrix_multiply !!!\n");
+	      fprintf(stderr, "overflow in SparseMatrix_multiply3 !!!\n");
 #endif
+	      free(mask);
 	      return NULL;
 	    }
-	    nz++;
 	    mask[jc[k]] = -i - 2;
 	  }
 	}
@@ -1140,7 +1089,6 @@ SparseMatrix SparseMatrix_multiply3(SparseMatrix A, SparseMatrix B, SparseMatrix
   }
 
   D = SparseMatrix_new(m, C->n, nz, type, FORMAT_CSR);
-  if (!D) goto RETURN;
   id = D->ia;
   jd = D->ja;
   
@@ -1174,7 +1122,6 @@ SparseMatrix SparseMatrix_multiply3(SparseMatrix A, SparseMatrix B, SparseMatrix
   
   D->nz = nz;
 
- RETURN:
   free(mask);
   return D;
 }
@@ -1374,10 +1321,8 @@ SparseMatrix SparseMatrix_remove_diagonal(SparseMatrix A){
     A->nz = nz;
     break;
   }
-  case MATRIX_TYPE_UNKNOWN:
-    return NULL;
   default:
-    return NULL;
+    UNREACHABLE();
   }
 
   return A;
@@ -1454,10 +1399,8 @@ SparseMatrix SparseMatrix_remove_upper(SparseMatrix A){/* remove diag and upper
     A->nz = nz;
     break;
   }
-  case MATRIX_TYPE_UNKNOWN:
-    return NULL;
   default:
-    return NULL;
+    UNREACHABLE();
   }
 
   A->is_pattern_symmetric = false;
@@ -1500,17 +1443,13 @@ SparseMatrix SparseMatrix_divide_row_by_degree(SparseMatrix A){
     }
     break;
   }
-  case MATRIX_TYPE_INTEGER:{
-    assert(0);/* this operation would not make sense for int matrix */
-    break;
-  }
+  case MATRIX_TYPE_INTEGER:
+    UNREACHABLE(); // this operation would not make sense for int matrix 
   case MATRIX_TYPE_PATTERN:{
     break;
   }
-  case MATRIX_TYPE_UNKNOWN:
-    return NULL;
   default:
-    return NULL;
+    UNREACHABLE();
   }
 
   return A;
@@ -1837,7 +1776,7 @@ SparseMatrix SparseMatrix_to_square_matrix(SparseMatrix A, int bipartite_options
   case BIPARTITE_ALWAYS:
     break;
   default:
-    assert(0);
+    UNREACHABLE();
   }
   B = SparseMatrix_get_augmented(A);
   SparseMatrix_delete(A);
@@ -1977,14 +1916,8 @@ SparseMatrix SparseMatrix_get_submatrix(SparseMatrix A, int nrow, int ncol, int
       }
     }
     break;
-  case MATRIX_TYPE_UNKNOWN:
-    free(rmask);
-    free(cmask);
-    return NULL;
   default:
-    free(rmask);
-    free(cmask);
-    return NULL;
+    UNREACHABLE();
   }
 
   B = SparseMatrix_from_coordinate_arrays(nz, nrow, ncol, irn, jcn, v, A->type, A->size);
